const express = require('express');

const transportApp = express.Router();

const libs = `${process.cwd()}/libs`;

const log = require(`${libs}/log`)(module);

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const multiQuery = require(`${libs}/middlewares/multiQuery`);

const response = require(`${libs}/middlewares/response`);

const id2str = require(`${libs}/middlewares/id2str`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const config = require(`${libs}/config`);

const cache = require('apicache').options({ debug: config.debug, statusCodes: {include: [200]} }).middleware;

let rqf = new ReqQueryFields();

transportApp.use(cache('15 day'));

transportApp.get('/year_range', (req, res, next) => {
    req.sql.from('matricula')
    .field('MIN(matricula.ano_censo)', 'start_year')
    .field('MAX(matricula.ano_censo)', 'end_year');
    next();
}, query, response('range'));

transportApp.get('/years', (req, res, next) => {
    req.sql.from('matricula')
    .field('DISTINCT matricula.ano_censo', 'year');
    next();
}, query, response('years'));

transportApp.get('/adm_dependency', (req, res, next) => {
    req.result = [];
    for(let i = 1; i <= 4; ++i) {
        req.result.push({
            id: i,
            name: id2str.admDependency(i)
        });
    };
    next();
}, response('adm_dependency'));

transportApp.get('/adm_dependency_detailed', (req, res, next) => {
    req.result = [];
    for(let i = 1; i <= 6; ++i) {
        req.result.push({
            id: i,
            name: id2str.admDependencyPriv(i)
        });
    };
    next();
}, response('adm_dependency_detailed'));

transportApp.get('/location', (req, res, next) => {
    req.result = [
        {id: 1, name: 'Urbana'},
        {id: 2, name: 'Rural'}
    ];
    next();
}, response('location'));

transportApp.get('/rural_location', (req, res, next) => {
    req.result = [
        {id: 1, name: "Urbana"},
        {id: 2, name: "Rural"},
        {id: 3, name: "Rural - Área de assentamento"},
        {id: 4, name: "Rural - Terra indígena"},
        {id: 5, name: "Rural - Área remanescente de quilombos"},
        {id: 6, name: "Rural - Unidade de uso sustentável"}
    ];
    next();
}, response('rural_location'));

transportApp.get('/education_level_basic', (req, res, next) => {
    req.result = [
        {id: null, name: 'Não classificada'},
        {id: 1, name: 'Creche'},
        {id: 2, name: 'Pré-Escola'},
        {id: 4, name: 'Ensino Fundamental - anos iniciais'},
        {id: 5, name: 'Ensino Fundamental - anos finais'},
        {id: 6, name: 'Ensino Médio'}
    ];
    next();
}, response('education_level_basic'));

transportApp.get('/transportation_manager', (req, res, next) => {
    req.result = [
        {id: null, name: 'Não classificada'},
        {id: 1, name: 'Estadual'},
        {id: 2, name: 'Municipal'},
    ];
    next();
}, response('transportation_manager'));

rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValue({
    name: 'rural_location',
    table: 'matricula',
    tableField: 'localidade_area_rural',
    resultField: 'rural_location_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'localidade_area_rural'
    }
}).addValue({
    name: 'location',
    table: 'matricula',
    tableField: 'localizacao_id',
    resultField: 'location_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'localizacao_id'
    }
}).addValue({
    name:'adm_dependency',
    table: 'matricula',
    tableField: 'dependencia_adm_id',
    resultField: 'adm_dependency_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'dependencia_adm_id'
    }
}).addValue({
    name: 'adm_dependency_detailed',
    table: 'matricula',
    tableField: 'dependencia_adm_priv',
    resultField: 'adm_dependency_detailed_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'dependencia_adm_priv'
    }
}).addValue({
    name: 'transportation_manager',
    table: 'matricula',
    tableField: 'responsavel_transp',
    resultField: 'transportation_manager',
    where: {
        relation: '=',
        type: 'integer',
        field: 'responsavel_transp'
    }
}).addValue({
    name: 'education_level_basic',
    table: 'matricula',
    tableField: 'etapas_mod_ensino_segmento_id',
    resultField: 'education_level_basic_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'etapas_mod_ensino_segmento_id'
    }
}).addValue({
    name: 'min_year',
    table: 'matricula',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        field: 'ano_censo'
    }
}).addValue({
    name: 'max_year',
    table: 'matricula',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        field: 'ano_censo'
    }
});

function matchQueries(queryTotal, queryPartial) {
    let match = [];
    queryTotal.forEach((result) => {
        let newObj = {};
        let keys = Object.keys(result);
        keys.forEach((key) => {
            newObj[key] = result[key];
        });
        let index = keys.indexOf('total');
        if(index > -1) keys.splice(index, 1);
        let objMatch = null;

        for(let i = 0; i < queryPartial.length; ++i) {
            let partial = queryPartial[i];
            let foundMatch = true;
            for(let j = 0; j < keys.length; ++j) {
                let key = keys[j];
                if(partial[key] !== result[key]) {
                    foundMatch = false;
                    break;
                }
            }
            if(foundMatch) {
                objMatch = partial;
                break;
            }
        }

        if(objMatch) {
            newObj.percentage = (objMatch.total / result.total) * 100;
            newObj.partial = objMatch.total;
            newObj.total = result.total
            match.push(newObj);
        }
    });

    return match;
}

transportApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    req.querySet = [];
    req.queryIndex = {};

    let allTransports = req.sql.clone()
    allTransports.field('COUNT(*)', 'total')
    .field("'Brasil'", 'name')
    .field('matricula.ano_censo', 'year')
    .from('matricula')
    .join('turma', null, 'matricula.turma_id = turma.id AND matricula.ano_censo = turma.ano_censo')
    .group('matricula.ano_censo')
    .order('matricula.ano_censo')
    .where('turma.tipo_turma_id <= 3');
    req.queryIndex.allTransports = req.querySet.push(allTransports) - 1;


    // Vans e Kombi
    let allVansAndKombi = allTransports.clone();
    allVansAndKombi.where('matricula.transporte_vans_kombi = 1');
    req.queryIndex.allVansAndKombi = req.querySet.push(allVansAndKombi) - 1;

    // Vans e Kombi
    let goVansAndKombi = allTransports.clone();
    goVansAndKombi.where('matricula.transporte_vans_kombi = 1 OR matricula.transporte_vans_kombi = 0');
    req.queryIndex.goVansAndKombi = req.querySet.push(goVansAndKombi) - 1;

    // Micro-ônibus
    let allMicroBus = allTransports.clone();
    allMicroBus.where('matricula.transporte_micro_onibus = 1 OR matricula.transporte_micro_onibus = 0');
    req.queryIndex.allMicroBus = req.querySet.push(allMicroBus) - 1;

    // Micro
    let goMicroBus = allTransports.clone();
    goMicroBus.where('matricula.transporte_micro_onibus = 1');
    req.queryIndex.goMicroBus = req.querySet.push(goMicroBus) - 1;

    // Ônibus
    let allBus = allTransports.clone();
    allBus.where('matricula.transporte_onibus = 1 OR matricula.transporte_onibus = 0');
    req.queryIndex.allBus = req.querySet.push(allBus) - 1;

    let goBus = allTransports.clone();
    goBus.where('matricula.transporte_onibus = 1');
    req.queryIndex.goBus = req.querySet.push(goBus) - 1;
    //
    // Bicicleta
    let goBikes = allTransports.clone();
    goBikes.where('matricula.transporte_bicicleta = 1');
    req.queryIndex.goBikes = req.querySet.push(goBikes) - 1;

    // Tração Animal
    let goAnimalTraction = allTransports.clone();
    goAnimalTraction.where('matricula.transporte_animal = 1');
    req.queryIndex.goAnimalTraction = req.querySet.push(goAnimalTraction) - 1;

    // Outro Veículo
    let goOtherVehicle = allTransports.clone();
    goOtherVehicle.where('matricula.transporte_outro = 1');
    req.queryIndex.goOtherVehicle = req.querySet.push(goOtherVehicle) - 1;

    // Aquaviário/ Embarcação (capacidade até 5 alunos)
    let goWaterway_5_Students = allTransports.clone();
    goWaterway_5_Students.where('matricula.transporte_embar_0_5 = 1');
    req.queryIndex.goWaterway_5_Students = req.querySet.push(goWaterway_5_Students) - 1;

    // Aquaviário/ Embarcação (capacidade de 5 até 15 alunos)
    let goWaterway_15_Students = allTransports.clone();
    goWaterway_15_Students.where('matricula.transporte_embar_5_15 = 1');
    req.queryIndex.goWaterway_15_Students = req.querySet.push(goWaterway_15_Students) - 1;

    // Aquaviário/ Embarcação (capacidade de 15 até 35 alunos)r
    let goWaterway_35_Students = allTransports.clone();
    goWaterway_35_Students.where('matricula.transporte_embar_15_35 = 1');
    req.queryIndex.goWaterway_35_Students = req.querySet.push(goWaterway_35_Students) - 1;

    // Aquaviário/ Embarcação (capacidade mais 35 alunos)
    let goWaterwayMoreThan_35 = allTransports.clone();
    goWaterwayMoreThan_35.where('matricula.transporte_embar_35 = 1');
    req.queryIndex.goWaterwayMoreThan_35 = req.querySet.push(goWaterwayMoreThan_35) - 1;

    // Trêm / Metrô
    let goSubwayAndTrain = allTransports.clone();
    goSubwayAndTrain.where('matricula.transporte_trem_metro = 1');
    req.queryIndex.goSubwayAndTrain = req.querySet.push(goSubwayAndTrain) - 1;

    next();
  }, multiQuery, (req, res, next) => {
    // let van_and_kombi = matchQueries(req.result[req.queryIndex.goVansAndKombi], req.result[req.queryIndex.allVansAndKombi]);
    // let microBus = matchQueries(req.result[req.queryIndex.goMicroBus], req.result[req.queryIndex.allMicroBus]);
    // let Bus = matchQueries(req.result[req.queryIndex.goBus], req.result[req.queryIndex.allBus]);
    let van_and_kombi = req.result[req.queryIndex.goVansAndKombi]
    let micro_bus = req.result[req.queryIndex.goMicroBus]
    let Bus = req.result[req.queryIndex.goBus]
    let bike = req.result[req.queryIndex.goBikes]
    let animal_traction = req.result[req.queryIndex.goAnimalTraction]
    let other_vehicle = req.result[req.queryIndex.goOtherVehicle]
    let waterway_5_Students = req.result[req.queryIndex.goWaterway_5_Students]
    let waterway_10_Students = req.result[req.queryIndex.goWaterway_10_Students]
    let waterway_15_Students = req.result[req.queryIndex.goWaterway_15_Students]
    let waterway_35_Students = req.result[req.queryIndex.goWaterway_35_Students]
    let waterway_More_Than_35 = req.result[req.queryIndex.goWaterwayMoreThan_35]
    let subway_and_train = req.result[req.queryIndex.goSubwayAndTrain]
    req.result = [{
          van_and_kombi,
          micro_bus,
          Bus,
          bike,
          animal_traction,
          other_vehicle,
          waterway_5_Students,
          waterway_10_Students,
          waterway_15_Students,
          waterway_35_Students,
          waterway_More_Than_35,
          subway_and_train

    }]
    next();
  }, response('transports'));

module.exports = transportApp;
